package ru.ifmo.paraschenko.algorithms.lca;

import ru.ifmo.paraschenko.objects.ISuffixTree;
import ru.ifmo.paraschenko.objects.ISuffixAutomaton;
import ru.ifmo.paraschenko.objects.automata.LinearSuffixAutomaton;
import ru.ifmo.paraschenko.algorithms.tools.SegmentTree;

import java.util.Iterator;
import java.util.Arrays;

/**
 * Description.
 *
 * @author Dmitry Paraschenko
 */
public class SuffixAutomatonLCA {
    private ISuffixAutomaton auto;

    private int globalTime;
    private int[] time2depth;
    private int[] enter;
    private int[] length2node;
    private SegmentTree segmentTree;

    int nodeCount;
    private int[] next;
    private int[] length;

    public SuffixAutomatonLCA(ISuffixAutomaton auto) {
//        System.out.println(auto);
        this.auto = auto;
        time2depth = new int[4 * auto.length() + 1];
        enter = new int[2 * auto.length() + 1];
        nodeCount = 0;
        length2node = new int[auto.length() + 1];
        next = new int[auto.getNodeCount()];
        length = new int[auto.getNodeCount()];
        Arrays.fill(next, -1);
        for (int i = 0; i < next.length; i++) {
            dfsPrepare(i);
        }
    }

    public void init() {
        dfs(0, 0);
        segmentTree = new SegmentTree(time2depth);
    }

    private int dfsPrepare(int state) {
        if (next[state] >= 0) {
            return next[state];
        }
        int count = 0;
        int nextState = -1;
        LinearSuffixAutomaton automaton = (LinearSuffixAutomaton) auto;
        for (int edge = automaton.first[state]; edge != -1; edge = automaton.next[edge]) {
//        for (Iterator<Integer> i = auto.getEdgesIterator(state); i.hasNext(); ) {
//            int edge = i.next();
            count++;
            if (count > 1) {
                break;
            }
            nextState = auto.getTarget(edge);
        }
        if (count == 1 && !auto.isFinish(state)) {
            next[state] = dfsPrepare(nextState);
            length[state] = 1 + length[nextState];
        } else {
            next[state] = state;
            length[state] = 0;
        }
        return next[state];
    }
    
    private void dfs(int state, int depth) {
        int node = nodeCount++;
        enter[node] = globalTime;
        time2depth[globalTime++] = depth;
//        System.out.println("ver " + node + ":  state " + state + " * depth " + depth);
        if (auto.isFinish(state)) {
            length2node[depth] = node;
        }
        LinearSuffixAutomaton automaton = (LinearSuffixAutomaton) auto;
        for (int edge = automaton.first[state]; edge != -1; edge = automaton.next[edge]) {
//        for (Iterator<Integer> i = auto.getEdgesIterator(state); i.hasNext(); ) {
//            int edge = i.next();
            int nextState = auto.getTarget(edge);
            dfs(dfsPrepare(nextState), depth + 1 + length[nextState]);
            time2depth[globalTime++] = depth;
        }
    }

    public int getLCA(int length1, int length2) {
        int time1 = enter[length2node[length1]];
        int time2 = enter[length2node[length2]];
        if (time1 > time2) {
            int tmp = time1;
            time1 = time2;
            time2 = tmp;
        }
        return segmentTree.getMin(time1, time2);
    }
}
