package ru.ifmo.paraschenko.algorithms.lca;

import ru.ifmo.paraschenko.objects.ISuffixTree;
import ru.ifmo.paraschenko.objects.trees.AbstractSuffixTree;
import ru.ifmo.paraschenko.algorithms.tools.SegmentTree;

import java.util.Iterator;

/**
 * Description.
 *
 * @author Dmitry Paraschenko
 */
public class SuffixTreeLCA {
    private ISuffixTree tree;

    private int globalTime;
    private int[] time2depth;
    private int[] enter;
    private int[] length2node;
    private SegmentTree segmentTree;

    public SuffixTreeLCA(ISuffixTree tree) {
//        System.out.println(tree);
        this.tree = tree;
        time2depth = new int[2 * tree.getNodeCount() - 1];
        enter = new int[tree.getNodeCount()];
        dfs(0, 0);
        length2node = new int[tree.length() + 1];
        int node = tree.process(tree.getString());
        for (int length = tree.length(); length >= 0; length--) {
            length2node[length] = node;
            node = tree.getSuffix(node);
        }
        segmentTree = new SegmentTree(time2depth);
    }

    private void dfs(int node, int depth) {
//        System.out.println("dfs(" + node + ", " + depth + ")");
        enter[node] = globalTime;
        time2depth[globalTime++] = depth;
        AbstractSuffixTree suffixTree = (AbstractSuffixTree) tree;
        for (int edge = suffixTree.first[node]; edge != -1; edge = suffixTree.next[edge]) {
//        for (Iterator<Integer> i = tree.getEdgesIterator(node); i.hasNext(); ) {
//            int edge = i.next();
            dfs(tree.getTarget(edge), depth + tree.getEnd(edge) - tree.getBegin(edge));
            time2depth[globalTime++] = depth;
        }
    }

    public int getLCA(int length1, int length2) {
        int time1 = enter[length2node[length1]];
        int time2 = enter[length2node[length2]];
        if (time1 > time2) {
            int tmp = time1;
            time1 = time2;
            time2 = tmp;
        }
        return segmentTree.getMin(time1, time2);
    }
}
