package ru.ifmo.paraschenko.objects;

import ru.ifmo.paraschenko.LargeInputSizeException;
import ru.ifmo.paraschenko.objects.trees.AutomatonSuffixTree;
import ru.ifmo.paraschenko.objects.trees.SuffixTrie;
import ru.ifmo.paraschenko.objects.trees.MinimizedSuffixTree;
import ru.ifmo.paraschenko.objects.trees.UkkonenSuffixTree;

/**
 * Description.
 *
 * @author Dmitry Paraschenko
 */
public class SuffixTreeFactoryBuilder {
    private static final SuffixTreeFactoryBuilder INSTANCE = new SuffixTreeFactoryBuilder();

    private static final ISuffixTreeFactory AUTOMATON_SUFFIX_TREE_FACTORY = new ISuffixTreeFactory() {
        public ISuffixTree newInstance(String string) throws LargeInputSizeException {
            return new AutomatonSuffixTree(string);
        }
    };

    private static final ISuffixTreeFactory MINIMIZED_SUFFIX_TREE_FACTORY = new ISuffixTreeFactory() {
        public ISuffixTree newInstance(String string) throws LargeInputSizeException {
            return new MinimizedSuffixTree(string);
        }
    };

    private static final ISuffixTreeFactory SUFFIX_TRIE_FACTORY = new ISuffixTreeFactory() {
        public ISuffixTree newInstance(String string) throws LargeInputSizeException {
            return new SuffixTrie(string);
        }
    };

    private static final ISuffixTreeFactory UKKONEN_SUFFIX_TREE_FACTORY = new ISuffixTreeFactory() {
        public ISuffixTree newInstance(String string) throws LargeInputSizeException {
            return new UkkonenSuffixTree(string);
        }
    };

    private SuffixTreeFactoryBuilder() {
    }

    public ISuffixTreeFactory getAutomatonSuffixTreeFactory() {
        return AUTOMATON_SUFFIX_TREE_FACTORY;
    }

    public ISuffixTreeFactory getMinimizedSuffixTreeFactory() {
        return MINIMIZED_SUFFIX_TREE_FACTORY;
    }

    public ISuffixTreeFactory getSuffixTrieFactory() {
        return SUFFIX_TRIE_FACTORY;
    }

    public ISuffixTreeFactory getUkkonenSuffixTreeFactory() {
        return UKKONEN_SUFFIX_TREE_FACTORY;
    }

    public static SuffixTreeFactoryBuilder getInstance() {
        return INSTANCE;
    }
}
