package ru.ifmo.paraschenko.objects.graphs;

import ru.ifmo.paraschenko.objects.IGraph;
import ru.ifmo.paraschenko.LargeInputSizeException;

import java.util.Arrays;
import java.util.Iterator;

/**
 * Description.
 *
 * @author Dmitry Paraschenko
 */
public class AdjacencyListGraph implements IGraph {
    private int nodeCount;
    private int edgeCount;
    public final int[] first;
    public final int[] next;
    public final char[] chr;
    private final int[] target;

    public AdjacencyListGraph(int verCnt, int edgeCnt) throws LargeInputSizeException {
        if (verCnt < 0 || verCnt > 11000000 || edgeCnt < 0 || edgeCnt > 11000000) {
            throw new LargeInputSizeException();
        }
        nodeCount = 0;
        edgeCount = 0;
        first = new int[verCnt];
        Arrays.fill(first, -1); //TODO: remove
        next = new int[edgeCnt];
        chr = new char[edgeCnt];
        target = new int[edgeCnt];
    }

    public int getNodeCount() {
        return nodeCount;
    }

    public int createNode() {
        first[nodeCount] = -1;
        return nodeCount++;
	}

    public int getEdgeCount() {
        return edgeCount;
    }

    public int getEdge(int ver, char ch) {
        for (int edge = first[ver]; edge != -1; edge = next[edge]) {
            if (chr[edge] == ch) {
                return edge;
            }
        }
        return -1;
    }

    public int addEdge(int src, char ch, int trg) {
        next[edgeCount] = first[src];
        chr[edgeCount] = ch;
        target[edgeCount] = trg;
        first[src] = edgeCount;
        return edgeCount++;
    }

    public int getTarget(int edge) {
        return target[edge];
    }

    public void setTarget(int edge, int target) {
        this.target[edge] = target;
    }

    public Iterator<Integer> getEdgesIterator(final int node) {
        return new Iterator<Integer>() {
            private int currentEdge = first[node];

            public boolean hasNext() {
                return currentEdge >= 0;
            }

            public Integer next() {
                int edge = currentEdge;
                currentEdge = next[currentEdge];
                return edge;
            }

            public void remove() {
            }
        };
    }
}
