package ru.ifmo.paraschenko.objects.trees;

import ru.ifmo.paraschenko.objects.automata.LinearSuffixAutomaton;
import ru.ifmo.paraschenko.LargeInputSizeException;

import java.util.*;

public final class AutomatonSuffixTree extends AbstractSuffixTree {
	private LinearSuffixAutomaton automaton;
	private int[] automatonEdges;
    private int[] longest;

    public AutomatonSuffixTree(String string, LinearSuffixAutomaton auto) throws LargeInputSizeException {
		super(string, 2 + 2 * string.length());
        automaton = auto;
		automatonEdges = new int[automaton.getNodeCount()];
        Arrays.fill(automatonEdges, -1);
		longest = new int[automaton.getNodeCount()];
        Arrays.fill(longest, -1);
        for (int i = 0; i < longest.length; i++) {
            getAutomatonEdge(i);
            getLongest(i);
        }
    }

    public void init() {
        dfs(0, 0);
    }

    public AutomatonSuffixTree(String string) throws LargeInputSizeException {
		this(string, new LinearSuffixAutomaton(string));
    }

	private void dfs(int node, int state) {
        if (automaton.isFinish(state)) {
            setFinish(node, true);
        }
        for (int edge = automaton.first[state]; edge != -1; edge = automaton.next[edge]) {
            int childState = automaton.getTarget(edge);
            int newState = getAutomatonEdge(childState);
            int newNode = createNode();
			addEdge(node, newNode, length() - getLongest(childState) - 1, length() - getLongest(newState));
			dfs(newNode, newState);
        }
    }

	private int getAutomatonEdge(int state) {
		if (automatonEdges[state] != -1) {
			return automatonEdges[state];
		}
        int count = 0;
        int nextState = -1;
        for (int edge = automaton.first[state]; edge != -1; edge = automaton.next[edge]) {
            int child = automaton.getTarget(edge);
            count++;
            nextState = child;
        }
        if (count == 1 && !automaton.isFinish(state)) {
            return automatonEdges[state] = getAutomatonEdge(nextState);
        }
        return automatonEdges[state] = state;
	}

	private int getLongest(int state) {
		if (longest[state] != -1) {
			return longest[state];
		}
        int edge = automaton.first[state];
        int result = edge == -1 ? 0 : 1 + getLongest(automaton.getTarget(edge));
		return longest[state] = result;
	}
}
