package ru.ifmo.paraschenko.objects.trees;

import ru.ifmo.paraschenko.LargeInputSizeException;

/**
 * Description.
 *
 * @author Dmitry Paraschenko
 */
public final class UkkonenSuffixTree extends AbstractSuffixTree {
    private int nextNode;
    private int nextBegin;
    private int nextEnd;

    public UkkonenSuffixTree(String string) throws LargeInputSizeException {
        super(string, 2 + 2 * string.length());
//        if (string.length() > 100000) {
//            throw new LargeInputSizeException();
//        }
        setFinish(0, true);
        if (string.length() == 0) {
            return;
        }
        int longestEdgeTarget = createNode();
        addEdge(0, longestEdgeTarget, 0, 1);
        int beginPos = 1;
        // s[0..1], s[1..1]
phase:
        for (int right = 1; right < length(); right++) {
            // s[?..right] -> s[?..right+1]
            updatePrevNode(longestEdgeTarget);
            incEnd(getParentEdge(longestEdgeTarget), 1);
            setSuffix(longestEdgeTarget, -1);
            for (int left = beginPos; left <= right; left++) {
                int node = nextNode;
                int b = nextBegin;
                int e = nextEnd;

                if (b == e) { // node
                    if (first[node] == -1) { // #1
                        updatePrevNode(node);
                        incEnd(getParentEdge(longestEdgeTarget), length() - right - 1);
                        setFinish(longestEdgeTarget, true);
                        longestEdgeTarget = node;
                        beginPos = left + 1;
                        incEnd(getParentEdge(node), 1);
                        setSuffix(node, -1);
                        continue;
                    }
                    if (getEdge(node, charAt(right)) == -1) { // #2
                        updatePrevNode(node);
                        addEdge(node, createNode(), right, right + 1);
                    } else { // #3
                        continue phase;
                    }
                } else { //edge
                    int edge = getEdge(node, charAt(b));
                    if (charAt(getBegin(edge) + e - b) != charAt(right)) { // #2
                        int mid = splitEdge(edge, e - b);
                        updatePrevNode(mid);
                        addEdge(mid, createNode(), right, right + 1);
                    } else { // #3
                        continue phase;
                    }
                }
            }
        }
        updatePrevNode(longestEdgeTarget);
        setFinish(longestEdgeTarget, true);
        int right = length();
        for (int left = beginPos; left <= right; left++) {
            int node = nextNode;
            int b = nextBegin;
            int e = nextEnd;
            if (b == e) { // node
                setFinish(node, true);
                updatePrevNode(node);
            } else { //edge
                int edge = getEdge(node, charAt(b));
                int mid = splitEdge(edge, e - b);
                setFinish(mid, true);
                updatePrevNode(mid);
            }
        }
    }

    private void incEnd(int edge, int increment) {
        setEnd(edge, getEnd(edge) + increment);
    }

    private void updatePrevNode(int node) {
        nextNode = getSuffix(node);
        if (nextNode == -1) {
            int edge = getParentEdge(node);
            nextNode = getSource(edge);
            nextBegin = getBegin(edge);
            nextEnd = getEnd(edge);
            if (nextNode == 0) {
                nextBegin++;
            } else {
                nextNode = getSuffix(nextNode);
            }
            while (nextBegin < nextEnd) {
                edge = getEdge(nextNode, charAt(nextBegin));
                int length = getEnd(edge) - getBegin(edge);
                if (nextEnd - nextBegin >= length) {
                    nextBegin += length;
                    nextNode = getTarget(edge);
                } else {
                    break;
                }
            }
        } else {
            nextBegin = nextEnd = -1;
        }
    }
}
